import java.security.MessageDigest;
import javax.crypto.*;
import javax.crypto.spec.SecretKeySpec;


public class EAPRadiusAttributes extends BasicPayloadObject {
	public int attrType;
    public int length;         
	public byte[] value = null; 
	protected boolean isLengthSet = false;

	public EAPRadiusAttributes(int attr) {
		this.attrType = attr;
	}
	
	public EAPRadiusAttributes(int attr, byte[] data) {
		this.attrType = attr;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			this.length = getSize();
			isLengthSet = true;
		}
	}
	
	public EAPRadiusAttributes(int attr, byte[] data, int len) {
		this.attrType = attr;
		if(data != null) {
			int curlen = data.length >= len ? len : data.length;
			value = new byte[curlen];
			System.arraycopy(data, 0, value, 0, curlen);
			this.length = getSize();
			isLengthSet = true;
		}
	}
	
	public EAPRadiusAttributes(byte[] data, int attr, int len, int pos) {
		this.attrType = attr;
		if((data != null)&&(len > 0)&&(pos > 0)) {
			int curlen = 0;
			if(pos <= data.length - 1) {
				curlen = data.length >= len + pos ? len : data.length - pos;
				value = new byte[curlen];
				System.arraycopy(data, pos, value, 0, curlen);
			}
			this.length = getSize();
			isLengthSet = true;
		}
	}
	
	public static EAPRadiusAttributes cloneAttribute(EAPRadiusAttributes attr) {
		EAPRadiusAttributes newAttr = null;
		if(attr != null)
			newAttr = new EAPRadiusAttributes(attr.attrType, attr.value);
		return newAttr;
	}
	
	public int getSize() {
		if(value != null) 
			return 2 + value.length;
		else
			return 2;
	}

	public void setLength() {
		length = (byte)getSize();
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = (byte)len;
		isLengthSet = true;
	}


// The password is first padded at the end with nulls to a multiple of 16 octets.
// A one-way MD5 hash is calculated over a stream of octets consisting of the shared secret followed by the Request Authenticator.
// This value is XORed with the first 16 octet segment of the password 
// and placed in the first 16 octets of the String field of the User-Password Attribute.

	public int setUserPassword(byte[] pwd, byte[] skey, byte[] reqAuth) throws Exception {
		MessageDigest mobj = null;
		byte[] pwdpad = null;  // password is first padded at the end with nulls
		byte[] sk_ra = null;   // shared key & Request Authenticator
		byte[] resMD5 = null;  // MD5 hash
		int res = 0;
		int i;
		
		if((pwd == null) || (skey == null) || (reqAuth == null)) {
			res = 1;
			DebugInfo.printMessage(3,"EAPRadiusAttributes::setUserPassword: null pointer");
		}

		i = pwd.length % 16;
		if(i != 0) {
			pwdpad = new byte[pwd.length + 16 - i];
			for(int j=0;j<pwdpad.length;j++)
				pwdpad[j] = 0;
			System.arraycopy(pwd, 0, pwdpad, 0, pwd.length);
		}
		else
			pwdpad = pwd;

		sk_ra = new byte[skey.length + reqAuth.length];
		System.arraycopy(skey, 0, sk_ra, 0, skey.length);
		System.arraycopy(reqAuth, 0, sk_ra, skey.length, reqAuth.length);

		DebugInfo.printDump(4, sk_ra, "setUserPassword sk_ra");

		mobj = MessageDigest.getInstance("MD5");
		resMD5 = mobj.digest(sk_ra);

		DebugInfo.printDump(4, resMD5, "setUserPassword resMD5");

		value = new byte[pwdpad.length];

		for(i=0;i<resMD5.length;i++) {
			value[i] = (byte)(pwdpad[i] ^ resMD5[i]);
		}

		DebugInfo.printDump(4, value, "setUserPassword value");

		return res;
	}


// shared secret followed by the Request Authenticator is put through a one-way MD5 hash
// to create a 16 octet digest value which is xored with the password entered by the user,
// and the xored result placed in the User-Password attribute in the Access-Request packet. (RFC 2865)
	public int setMessageAuthenticator(EAPRadiusHeader radPkt, byte[] key) {
		Mac mobj;
		SecretKeySpec sk;
		int res = 0;
		
		if((radPkt == null) || (key == null)) {
			res = 1;
			DebugInfo.printMessage(3,"EAPRadiusAttributes::setMessageAuthenticator: null pointer");
		}

		if(value == null)
			value = new byte[16];
		for(int j=0;j<value.length;j++)
			value[j] = 0;

		this.toByteArray();
		radPkt.toByteArray();
		DebugInfo.printDump(4, radPkt.net_msg, "EAPRadiusAttributes::setMessageAuthenticator: packet:");
			
		try {
			mobj = Mac.getInstance("HmacMD5");
			sk = new SecretKeySpec(key, "HmacMD5");
			mobj.init(sk);
			value = mobj.doFinal(radPkt.net_msg);
			this.isLengthSet = false;
		}  catch (Exception err) {
			DebugInfo.printMessage(1,"EAPRadiusAttributes:setMessageAuthenticator: " + err);
		}

		this.toByteArray();
		radPkt.isLengthSet = false;

		return res;
	}

	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)attrType;
		ba[i++] = (byte)length;
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
		}
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + this.attrType);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printDump(1, this.value, pref + "  value:");
	}
}    
